#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QTcpServer>
#include <QTcpSocket>
#include <QXmlSimpleReader>
#include <QSettings>

#include <lfrxmlhandler.h>
#include <lfrxmlwriter.h>
#include <mainwindowui.h>
#include "tmpackettoread.h"
#include "wfpacket.h"
#include "lfrxmlparser.h"

class MainWindow : public QWidget
{
    Q_OBJECT
    
public:
    MainWindow(QWidget *parent = 0);
    ~MainWindow();

    void displayNetworkInterfaces();
    void initSocketStatesList();
    void preProcessPacket(TMPacketToRead *packet);
    void buildWFAndDisplay(TMPacketToRead *packet, WFPacket *wfPacket, unsigned char num_page);
    void buildWFAndDisplayBurst(TMPacketToRead *packet, WFPacket *wfPacket);
    void buildWFAndDisplaySBM1(TMPacketToRead *packet, WFPacket *wfPacket);
    void buildWFAndDisplaySBM2(TMPacketToRead *packet, WFPacket *wfPacket);

protected:
    void closeEvent(QCloseEvent *);

private:
    void readSettings();
    void writeSettings();
    unsigned char echoBridgeIP0;
    unsigned char echoBridgeIP1;
    unsigned char echoBridgeIP2;
    unsigned char echoBridgeIP3;

    QByteArray dataArray;

    QList<TMPacketToRead*> generalCCSDSPacketStore;

    QTcpServer *analyserSGSEServerTC;
    QTcpServer *analyserSGSEServerTM;
    QTcpSocket *socketTC;
    QTcpSocket *socketTM;
    QTcpSocket *socketEchoServer;

    unsigned int totalOfBytes;
    unsigned int totalOfPackets;

    QXmlSimpleReader xmlReader;
    QXmlInputSource *sourceTC;
    LFRXmlHandler *xmlHandler;
    LFRXmlParser *lfrXmlParser;

    QList<QString> socketStates;

    char *buffer;

    MainWindowUI *UI;

    LFRXmlWriter *lfrXmlWriter;

    WFPacket wfPacketNormal[4];
    WFPacket wfPacketBurst;
    WFPacket wfPacketSBM1;
    WFPacket wfPacketSBM2;

    bool parsingContinue;

signals:
    void socketTMHasChanged(QTcpSocket *socket);

public slots:
    void displayOnConsole(QString message);
    void listenOnTCTMPorts();
    void testTCTMPorts();
    void newConnectionOnTCServer();
    void newConnectionOnTMServer();
    void readDataOnTCPort();
    void readDataOnTMPort();
    void readDataOnSocketEchoServer();
    void openEchoServer();
    void socket_TMEcho_ServerHasChanged();
    void processIncomingPacket(TMPacketToRead *packet);
    void resetStatistics();
    void processPacketStore();
};

#endif // MAINWINDOW_H
